from enum import IntEnum


class Element(str):
    def __new__(cls, value: str, name: str = None):
        instance = super().__new__(cls, value)
        if name is None:
            instance.name = value
        else:
            instance.name = name

        return instance


class ExitCode(IntEnum):
    SUCCESS = 0
    DECODING_ERROR = 1
    INVALID_SELECTION = 2



import os
import shutil
import sys
from pathlib import Path

TESTING_DIR: Path = Path(os.curdir + "testing").absolute()
PROTON_DIR_1: Path = Path(TESTING_DIR / "proton_dir_1")
PROTON_DIR_2: Path = Path(TESTING_DIR / "proton_dir_2")
USER_DIR: Path = Path(TESTING_DIR / "user_dir")

PROTON_BIG: Path = PROTON_DIR_1 / "UMU_Proton_10"
PROTON_SMALL: Path = PROTON_DIR_1 / "UMU_Proton_1"

sys.path.insert(1, os.path.join(os.path.abspath(os.curdir), "src"))


def teardown():
    shutil.rmtree(TESTING_DIR)


def setup():
    if TESTING_DIR.exists():
        teardown()

    TESTING_DIR.mkdir()
    PROTON_DIR_1.mkdir()
    PROTON_BIG.mkdir()
    PROTON_SMALL.mkdir()
    PROTON_DIR_2.mkdir()
    USER_DIR.mkdir()



import unittest

import umu_commander.configuration as config
from tests import *
from umu_commander import proton


class Tracking(unittest.TestCase):
    def setUp(self):
        config.PROTON_PATHS = [PROTON_DIR_1, PROTON_DIR_2]
        config.UMU_PROTON_PATH = PROTON_DIR_1
        setup()

    def tearDown(self):
        teardown()

    def test_collect_proton_versions(self):
        with open(os.path.join(PROTON_DIR_1, config.DB_NAME), "wt") as file:
            file.write("Must be ignored.")
        versions = proton.collect_proton_versions()
        self.assertTrue(
            len([*versions[PROTON_DIR_1]]) == 2 and versions.get(PROTON_DIR_2) is None,
            "Did not collect proton versions correctly.",
        )

    def test_get_latest_umu_proton(self):
        latest: Path = proton.get_latest_umu_proton()
        self.assertEqual(latest, PROTON_BIG, "Deduced latest proton incorrectly.")



import unittest

import umu_commander.configuration as config
from tests import *
from umu_commander import configuration


class Config(unittest.TestCase):
    def setUp(self):
        configuration.CONFIG_DIR = TESTING_DIR
        configuration.DB_DIR = TESTING_DIR
        setup()

    def tearDown(self):
        teardown()

    def test_missing_config(self):
        config.dump()
        self.assertTrue((TESTING_DIR / configuration.CONFIG_NAME).exists())
        config.load()



import shutil
from pathlib import Path

from InquirerPy import inquirer

import umu_commander.database as db
from umu_commander.configuration import DEFAULT_UMU_CONFIG_NAME
from umu_commander.proton import (
    collect_proton_versions,
    get_latest_umu_proton,
    refresh_proton_versions,
)
from umu_commander.util import (
    build_choices,
)


def select_config() -> str:
    files = [file for file in Path.cwd().iterdir() if file.is_file()]
    choices = build_choices(files, None)
    return inquirer.select("Select umu-commander config:", choices).execute()


def untrack(target_dir: Path = None, *, quiet: bool = False):
    if target_dir is None:
        target_dir = Path.cwd()

    target_dir = target_dir.absolute()

    for proton_dir in db.get().keys():
        for proton_ver in db.get(proton_dir):
            if target_dir in db.get(proton_dir, proton_ver):
                db.get(proton_dir, proton_ver).remove(target_dir)

    if not quiet:
        print("Config removed from all tracking lists.")


def track(
    proton_ver: Path = None,
    config: Path = None,
    *,
    interactive: bool = True,
    refresh_versions: bool = True,
    quiet: bool = False,
):
    if config is None:
        if interactive:
            config = select_config()

        else:
            config = Path.cwd() / DEFAULT_UMU_CONFIG_NAME

    if refresh_versions:
        refresh_proton_versions()

    if proton_ver is None:
        proton_dirs = collect_proton_versions(sort=True)
        choices = build_choices(None, proton_dirs)
        proton_ver: Path = inquirer.select(
            "Select Proton version to track config with:", choices
        ).execute()

    proton_ver = proton_ver.absolute()
    config = config.absolute()

    untrack(config, quiet=True)
    db.get(proton_ver.parent, proton_ver).append(config)

    if not quiet:
        print(
            f"Config {config} added to Proton version's {proton_ver.name} in {proton_ver.parent} tracking list."
        )


def users(proton_ver: Path = None):
    if proton_ver is None:
        proton_dirs = collect_proton_versions(sort=True)
        choices = build_choices(None, proton_dirs, count_elements=True)
        proton_ver: Path = inquirer.select(
            "Select Proton version to view user list:", choices
        ).execute()

    proton_ver = proton_ver.absolute()

    if proton_ver.parent in db.get() and proton_ver in db.get(proton_ver.parent):
        version_users: list[Path] = db.get(proton_ver.parent, proton_ver)
        if len(version_users) > 0:
            print(
                f"Directories tracked by {proton_ver.name} of {proton_ver.parent}:",
                *version_users,
                sep="\n\t",
            )

        else:
            print("This version is tracking no configs.")

    else:
        print("This version hasn't been used by umu before.")


def delete():
    for proton_dir in db.get().keys():
        for proton_ver, version_users in db.get(proton_dir).copy().items():
            if proton_ver == get_latest_umu_proton():
                continue

            if len(version_users) == 0:
                confirmed: bool = inquirer.confirm(
                    f"Version {proton_ver.name} in {proton_dir} is tracking no directories, delete?"
                ).execute()
                if confirmed:
                    try:
                        shutil.rmtree(proton_dir / proton_ver)
                    except FileNotFoundError:
                        pass
                    del db.get(proton_dir)[proton_ver]


def untrack_unlinked():
    for proton_dir in db.get().keys():
        for proton_ver, version_users in db.get()[proton_dir].items():
            for user in version_users:
                if not user.exists():
                    db.get(proton_dir, proton_ver).remove(user)



from collections.abc import Iterable
from pathlib import Path

from InquirerPy.base.control import Choice
from InquirerPy.separator import Separator

from umu_commander import database as db
from umu_commander.Types import Element


def count_users(proton_dir: Path, proton_ver: Path) -> str:
    return (
        f"({len(db.get(proton_dir, proton_ver))})"
        if proton_ver in db.get(proton_dir)
        else "(-)"
    )


def build_choices(
    elements: Iterable[Path | Element] | None,
    groups: dict[Path | Element, Iterable[Path | Element]] | None,
    *,
    count_elements: bool = False,
) -> list[Separator | Choice | str]:
    if elements is None:
        elements = []

    if groups is None:
        groups = {}

    choices: list[Choice | Separator] = [Choice(el, name=el.name) for el in elements]
    if len(choices) > 0:
        choices.append(Separator(""))

    for group, elements in groups.items():
        choices.extend(
            [
                Separator(f"In: {group}"),
                *[
                    Choice(
                        el,
                        name=(
                            el.name
                            if not count_elements
                            else f"{el.name} {count_users(group, el)}"
                        ),
                    )
                    for el in elements
                ],
                Separator(""),
            ]
        )

    return choices



import argparse
from argparse import ArgumentParser, Namespace
from json import JSONDecodeError
from pathlib import Path

from InquirerPy.exceptions import InvalidArgument

from umu_commander import VERSION
from umu_commander import configuration as config
from umu_commander import database as db
from umu_commander import tracking, umu_config
from umu_commander.configuration import CONFIG_DIR, CONFIG_NAME, DEFAULT_UMU_CONFIG_NAME
from umu_commander.Types import ExitCode


def init() -> ExitCode:
    try:
        config.load()

    except (JSONDecodeError, KeyError):
        config_path: Path = CONFIG_DIR / CONFIG_NAME
        config_path_old: Path = CONFIG_DIR / (str(CONFIG_NAME) + ".old")

        print(f"Config file at {config_path} could not be read.")

        if not config_path_old.exists():
            print(f"Config file renamed to {config_path_old}.")
            config_path.rename(config_path_old)

        return ExitCode.DECODING_ERROR

    except FileNotFoundError:
        config.dump()

    try:
        db.load()

    except JSONDecodeError:
        db_path: Path = config.DB_DIR / config.DB_NAME
        db_path_old: Path = config.DB_DIR / (str(config.DB_NAME) + ".old")

        print(f"Tracking file at {db_path} could not be read.")

        if not db_path_old.exists():
            db_path.rename(db_path_old)
            print(f"DB file renamed to {db_path_old}.")

    except FileNotFoundError:
        pass

    return ExitCode.SUCCESS


def get_parser_results() -> tuple[ArgumentParser, Namespace]:
    parser = argparse.ArgumentParser(
        prog=f"umu-commander",
        description="Interactive CLI tool to augment umu-launcher",
        epilog="For details, usage, and more, see the README.md file, or visit https://github.com/Mpaxlamitsounas/umu-commander.",
        add_help=True,
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s {VERSION}",
        help="Prints this menu.",
    )
    parser.add_argument(
        "-q",
        "--quiet",
        action="store_true",
        help="Suppresses all non interactive output.",
    )
    parser.add_argument(
        "verb",
        help="Selects program functionality.",
        choices=["track", "untrack", "users", "delete", "create", "run", "fix"],
    )
    parser.add_argument(
        "-i",
        "--input",
        help=f'Path to file or directory that will be acted upon. Default: $PWD for directories, "{DEFAULT_UMU_CONFIG_NAME}" for files',
        type=Path,
    )
    parser.add_argument(
        "-pr",
        "--proton",
        help="Path to Proton version that will be used. Default: Latest umu proton",
        type=Path,
    )
    parser.add_argument(
        "-wp",
        "--wine-prefix",
        help='Sets the WINE prefix path in config creation. Default: directory/"prefix"',
        type=Path,
        dest="prefix",
    )
    parser.add_argument(
        "-dll",
        "--dll-overrides",
        help="Sets the WINE DLL override string in config creation. Default: No overrides",
    )
    parser.add_argument(
        "-l",
        "--lang",
        help="Sets the language locale override in config creation. Default: No override",
    )
    parser.add_argument(
        "-a",
        "--launch-args",
        help='Sets launch arguments in config creation. Must be supplied as -a/--launch-args="<Launch args>" (with an equals sign) because of argparse limitations. Default: No arguments',
    )
    parser.add_argument(
        "-r",
        "--runners",
        help="Sets runners (like MangoHud) in config creation, order matters. Default: No runners",
    )
    parser.add_argument(
        "-o",
        "--output",
        help=f"Sets output config filename in config creation. Default directory/{DEFAULT_UMU_CONFIG_NAME}",
        type=Path,
    )
    parser.add_argument(
        "-ni",
        "--no-interactive",
        help="Uses defaults for missing values instead of prompting.",
        action="store_false",
        dest="interactive",
    )
    parser.add_argument(
        "-nr",
        "--no-refresh",
        "-nu",
        "--no-update",
        help="Do not check for new Proton versions while running.",
        action="store_false",
        dest="refresh",
    )

    args = parser.parse_args()

    args.launch_args = (
        args.launch_args.split(" ") if args.launch_args is not None else None
    )
    args.runners = args.runners.split(" ") if args.runners is not None else None

    return parser, args


def main() -> int:
    if (return_code := init()) != ExitCode.SUCCESS:
        return return_code.value

    parser, args = get_parser_results()

    try:
        match args.verb:
            case "track":
                tracking.track(
                    args.proton,
                    args.input,
                    refresh_versions=args.refresh,
                    quiet=args.quiet,
                )

            case "untrack":
                tracking.untrack(args.input, quiet=args.quiet)

            case "users":
                tracking.users(args.proton)

            case "delete":
                tracking.delete()

            case "create":
                umu_config.create(
                    args.prefix,
                    args.proton,
                    args.dll_overrides,
                    args.lang,
                    args.launch_args,
                    args.runners,
                    args.input,
                    args.output,
                    interactive=args.interactive,
                    refresh_versions=args.refresh,
                    quiet=args.quiet,
                )

            case "run":
                umu_config.run(args.input)

            case "fix":
                umu_config.fix(args.input)

            case _:
                print("Unrecognised verb.")
                parser.print_help()
                return ExitCode.INVALID_SELECTION.value

    except InvalidArgument:
        print("No choices to select from.")
        return ExitCode.INVALID_SELECTION.value

    else:
        return ExitCode.SUCCESS.value

    finally:
        tracking.untrack_unlinked()
        db.dump()


if __name__ == "__main__":
    exit(main())



import json
from collections import defaultdict
from pathlib import Path

import umu_commander.configuration as config

_db: defaultdict[Path, defaultdict[Path, list[Path]]] = defaultdict(
    lambda: defaultdict(list)
)


def load():
    global _db

    if not config.DB_DIR.exists():
        config.DB_DIR.mkdir()

    with open(config.DB_DIR / config.DB_NAME, "rt") as db_file:
        db: dict[Path, dict[Path, list[Path]]] = {}
        for proton_dir, proton_vers in json.load(db_file).items():
            proton_dir = Path(proton_dir)
            db[proton_dir] = {}
            for proton_ver, proton_users in proton_vers.items():
                proton_ver = proton_dir / proton_ver
                db[proton_dir][proton_ver] = [Path(user) for user in proton_users]
        # noinspection PyTypeChecker
        _db.update(db)


def dump():
    if not config.DB_DIR.exists():
        config.DB_DIR.mkdir()

    db: dict[str, dict[str, list[str]]] = {}
    for proton_dir, proton_vers in _db.items():
        proton_dir = str(proton_dir)
        db[proton_dir] = {}
        for proton_ver, proton_users in proton_vers.items():
            proton_ver = proton_ver.name
            db[proton_dir][proton_ver] = list(
                dict.fromkeys([str(user) for user in proton_users])
            )

    with open(config.DB_DIR / config.DB_NAME, "wt") as db_file:
        # noinspection PyTypeChecker
        json.dump(db, db_file, indent="\t")


def get(
    proton_dir: Path = None, proton_ver: Path = None
) -> dict[Path, dict[Path, list[Path]]] | dict[Path, list[Path]] | list[Path]:
    global _db

    if proton_dir is None and proton_ver is None:
        return _db

    if proton_ver is None:
        return _db[proton_dir]

    if proton_ver not in _db[proton_dir]:
        _db[proton_dir][proton_ver] = []

    return _db[proton_dir][proton_ver]


def _reset():
    global _db
    _db = defaultdict(lambda: defaultdict(list))



import os
import subprocess
import tomllib
from collections.abc import Iterable
from pathlib import Path
from typing import Any

import tomli_w
from InquirerPy import inquirer

import umu_commander.configuration as config
from umu_commander import database as db
from umu_commander import tracking
from umu_commander.configuration import (
    DEFAULT_UMU_CONFIG_NAME,
    DLL_OVERRIDES_OPTIONS,
    LANG_OVERRIDES_OPTIONS,
)
from umu_commander.proton import (
    collect_proton_versions,
    get_latest_umu_proton,
    refresh_proton_versions,
)
from umu_commander.Types import Element
from umu_commander.util import build_choices


def select_prefix() -> Path:
    default = Element(str(Path.cwd() / "prefix"), "Current directory")
    choices = build_choices([default, *config.DEFAULT_PREFIX_DIR.iterdir()], None)
    return inquirer.select("Select wine prefix:", choices, default).execute()


def select_proton() -> Path:
    choices = build_choices(None, collect_proton_versions(sort=True))
    return inquirer.select(
        "Select Proton version:", choices, Path.cwd() / "prefix"
    ).execute()


def select_dll_override() -> str:
    choices = build_choices(DLL_OVERRIDES_OPTIONS, None)
    return "".join(
        [
            selection
            for selection in inquirer.checkbox(
                "Select DLLs to override:", choices
            ).execute()
        ]
    )


def select_lang() -> str:
    default = Element("", "No override")
    choices = build_choices([default, *LANG_OVERRIDES_OPTIONS], None)
    return inquirer.select("Select locale:", choices, default).execute()


def set_launch_args() -> list[str]:
    options: str = inquirer.text(
        "Enter executable options, separated by space:"
    ).execute()
    return [opt.strip() for opt in options.split(" ") if opt.strip() != ""]


def set_runners() -> list[str]:
    options: str = inquirer.text(
        "Enter runners in order, separated by space:"
    ).execute()
    return [opt.strip() for opt in options.split(" ") if opt.strip() != ""]


def select_exe() -> Path:
    files = [file for file in Path.cwd().iterdir() if file.is_file()]
    choices = build_choices(files, None)
    return inquirer.select("Select game executable:", choices).execute()


def create(
    prefix: Path = None,
    proton_ver: Path = None,
    dll_overrides: str = None,
    lang: str = None,
    launch_args: Iterable[str] = None,
    runners: Iterable[str] = None,
    exe: Path = None,
    output: Path = None,
    *,
    interactive: bool = True,
    refresh_versions: bool = True,
    quiet: bool = False,
):
    if refresh_versions:
        refresh_proton_versions()

    # Prefix selection
    if prefix is None:
        if interactive:
            prefix = select_prefix()

        else:
            prefix = Path.cwd() / "prefix"

    # Proton selection
    if proton_ver is None:
        if interactive:
            proton_ver = select_proton()

        else:
            proton_ver = get_latest_umu_proton()

    # Select DLL overrides
    if dll_overrides is None and interactive:
        dll_overrides = select_dll_override()

    # Set language locale
    if lang is None and interactive:
        lang = select_lang()

    # Input executable launch args
    if launch_args is None and interactive:
        launch_args = set_launch_args()

    # Set runners for umu-run
    if runners is None and interactive:
        runners = set_runners()

    # Select executable name
    if exe is None:
        exe = select_exe()

    params: dict[str, Any] = {
        "umu": {
            "prefix": str(prefix),
            "proton": str(proton_ver),
            "runners": runners,
            "exe": str(exe),
            "launch_args": launch_args,
        },
        "env": {"WINEDLLOVERRIDES": dll_overrides, "LANG": lang},
    }

    if not params["umu"]["launch_args"]:
        del params["umu"]["launch_args"]

    if not params["env"]["WINEDLLOVERRIDES"]:
        del params["env"]["WINEDLLOVERRIDES"]

    if not params["env"]["LANG"]:
        del params["env"]["LANG"]

    if not params["umu"]["runners"]:
        del params["umu"]["runners"]

    if output is None:
        output = Path.cwd() / DEFAULT_UMU_CONFIG_NAME

    try:
        with open(output, "wb") as file:
            tomli_w.dump(params, file)

    except (ValueError, TypeError):
        if not quiet:
            print("Could not create configuration file.")

    if not quiet:
        print(f"Configuration file {output.name} created in {output.parent}.")
        print(f"Use with umu-commander run.")

    tracking.track(proton_ver, output, refresh_versions=False, quiet=quiet)


def run(umu_config: Path = None):
    if umu_config is None:
        umu_config = Path.cwd() / DEFAULT_UMU_CONFIG_NAME

    if not umu_config.exists():
        print("Specified umu config does not exist.")
        return

    with open(umu_config, "rb") as toml_file:
        toml_conf = tomllib.load(toml_file)

        prefix_path = Path(toml_conf["umu"]["prefix"])
        if not prefix_path.exists():
            prefix_path.mkdir()

        os.environ.update(toml_conf.get("env", {}))
        subprocess.run(
            args=[
                *toml_conf["umu"].get("runners", []),
                "umu-run",
                "--config",
                umu_config,
            ],
            env=os.environ,
        )


def fix(umu_config: Path = None):
    if umu_config is None:
        umu_config = Path.cwd() / DEFAULT_UMU_CONFIG_NAME

    if not umu_config.exists():
        print("Specified umu config does not exist.")
        return

    with open(umu_config, "rb") as toml_file:
        toml_conf = tomllib.load(toml_file)

    base_dir = umu_config.parent
    proton = Path(toml_conf["umu"]["proton"])
    prefix = Path(toml_conf["umu"]["prefix"])
    db.get(proton.parent, proton).append(umu_config)
    if not prefix.exists() and prefix.is_absolute():
        toml_conf["umu"]["prefix"] = str(base_dir / prefix.name)

    exe = Path(toml_conf["umu"]["exe"])
    if not exe.exists() and exe.is_absolute():
        toml_conf["umu"]["exe"] = str(base_dir / exe.name)

    with open(umu_config, "wb") as toml_file:
        tomli_w.dump(toml_conf, toml_file)



import importlib
import os
import tomllib
from pathlib import Path
from typing import Any

import tomli_w

from umu_commander.Types import Element

CONFIG_DIR: Path = Path.home() / ".config"
CONFIG_NAME: Path = Path("umu-commander.toml")

PROTON_PATHS: tuple[Path, ...] = (
    Path.home() / ".local/share/Steam/compatibilitytools.d",
    Path.home() / ".local/share/umu/compatibilitytools",
    Path.home()
    / ".var/app/com.valvesoftware.Steam/.local/share/Steam/compatibilitytools.d",
)
UMU_PROTON_PATH: Path = Path(Path.home() / ".local/share/Steam/compatibilitytools.d")

DB_NAME: Path = Path("tracking.json")
DB_DIR: Path = Path.home() / ".local/share/umu/compatibilitytools"
DEFAULT_UMU_CONFIG_NAME: Path = Path("umu-config.toml")
DEFAULT_PREFIX_DIR: Path = Path.home() / ".local/share/wineprefixes/"
DLL_OVERRIDES_OPTIONS: tuple[Element, ...] = (
    Element("winhttp.dll=n,b;", "winhttp for BepInEx"),
)
LANG_OVERRIDES_OPTIONS: tuple[Element, ...] = (Element("ja_JP.UTF8", "Japanese"),)

module = importlib.import_module(__name__)


def load():
    with open(os.path.join(CONFIG_DIR, CONFIG_NAME), "rb") as conf_file:
        toml_conf = tomllib.load(conf_file)

        # Proton dirs translation
        setattr(
            module,
            "PROTON_PATHS",
            (Path(proton_dir) for proton_dir in toml_conf["PROTON_PATHS"]),
        )
        del toml_conf["PROTON_PATHS"]

        # DLL/LANG Override translation
        for key in ["DLL", "LANG"]:
            setattr(
                module,
                f"{key}_OVERRIDES_OPTIONS",
                (
                    Element(value, name)
                    for name, value in toml_conf[f"{key}_OVERRIDES_OPTIONS"].items()
                ),
            )
            del toml_conf[f"{key}_OVERRIDES_OPTIONS"]

        for key, value in toml_conf.items():
            setattr(module, key, Path(value))


def _get_attributes() -> dict[str, Any]:
    attributes: dict[str, Any] = {}
    for key in dir(module):
        value = getattr(module, key)
        if not key.startswith("_") and not callable(value) and key.upper() == key:
            attributes[key] = value

    return attributes


def dump():
    if not os.path.exists(CONFIG_DIR):
        os.mkdir(CONFIG_DIR)

    with open(os.path.join(CONFIG_DIR, CONFIG_NAME), "wb") as conf_file:
        toml_conf = _get_attributes()
        del toml_conf["CONFIG_DIR"]
        del toml_conf["CONFIG_NAME"]

        for key, value in toml_conf.items():
            match key:
                case "PROTON_PATHS":
                    toml_conf[key] = [str(proton_dir) for proton_dir in PROTON_PATHS]

                case "DLL_OVERRIDES_OPTIONS" | "LANG_OVERRIDES_OPTIONS":
                    toml_conf[key] = {
                        override.name: override for override in getattr(module, key)
                    }

                case _:
                    toml_conf[key] = str(value)

        tomli_w.dump(toml_conf, conf_file)



import re
import subprocess
from collections.abc import Iterable
from pathlib import Path

import umu_commander.configuration as config


def _natural_sort_proton_ver_key(p: Path, _nsre=re.compile(r"(\d+)")):
    s: str = p.name
    return [int(text) if text.isdigit() else text for text in _nsre.split(s)]


def refresh_proton_versions():
    print("Updating umu Proton.")
    umu_update_process = subprocess.run(
        ["umu-run", '""'],
        env={"PROTONPATH": "UMU-Latest", "UMU_LOG": "debug"},
        capture_output=True,
        text=True,
    )

    for line in umu_update_process.stderr.split("\n"):
        if "PROTONPATH" in line and "/" in line:
            try:
                left: int = line.rfind("/") + 1
                print(f"Latest UMU-Proton: {line[left:len(line) - 1]}.")
            except ValueError:
                print("Could not fetch latest UMU-Proton.")

            break


def collect_proton_versions(sort: bool = False) -> dict[Path, Iterable[Path]]:
    versions: dict[Path, Iterable[Path]] = {}
    for proton_dir in config.PROTON_PATHS:
        dir_versions = [version for version in proton_dir.iterdir() if version.is_dir()]
        if len(dir_versions) == 0:
            continue
        else:
            versions[proton_dir] = dir_versions

        if sort:
            versions[proton_dir] = sorted(
                versions[proton_dir], key=_natural_sort_proton_ver_key, reverse=True
            )

    return versions


def get_latest_umu_proton() -> Path | None:
    umu_proton_versions: list[Path] = [
        config.UMU_PROTON_PATH / proton_ver
        for proton_ver in config.UMU_PROTON_PATH.iterdir()
        if "UMU" in proton_ver.name and (config.UMU_PROTON_PATH / proton_ver).is_dir()
    ]

    if len(umu_proton_versions) == 0:
        return None

    umu_proton_versions = sorted(
        umu_proton_versions, key=_natural_sort_proton_ver_key, reverse=True
    )

    return umu_proton_versions[0]



import unittest

import umu_commander.configuration as config
import umu_commander.database as db
from tests import *
from umu_commander import tracking
from umu_commander.configuration import DEFAULT_UMU_CONFIG_NAME


class Tracking(unittest.TestCase):
    def setUp(self):
        config.DB_DIR = TESTING_DIR
        setup()

    def tearDown(self):
        teardown()

    def test_track_untrack(self):
        os.chdir(USER_DIR)

        tracking.track(
            PROTON_DIR_1 / PROTON_BIG, DEFAULT_UMU_CONFIG_NAME, refresh_versions=False
        )

        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertIn(
            USER_DIR / DEFAULT_UMU_CONFIG_NAME, db.get(PROTON_DIR_1, PROTON_BIG)
        )

        tracking.untrack(quiet=True)
        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertNotIn(USER_DIR, db.get(PROTON_DIR_1, PROTON_BIG))

    def test_track_auto_untrack(self):
        os.chdir(USER_DIR)

        tracking.track(
            PROTON_DIR_1 / PROTON_BIG, DEFAULT_UMU_CONFIG_NAME, refresh_versions=False
        )
        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertIn(
            USER_DIR / DEFAULT_UMU_CONFIG_NAME, db.get(PROTON_DIR_1, PROTON_BIG)
        )

        USER_DIR.rmdir()
        tracking.untrack_unlinked()
        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertNotIn(
            USER_DIR,
            db.get(PROTON_DIR_1, PROTON_BIG),
            "Auto untrack did not untrack removed directory.",
        )



import unittest
from json import JSONDecodeError

import umu_commander.configuration as config
import umu_commander.database as db
from tests import *


class Database(unittest.TestCase):
    def setUp(self):
        config.DB_DIR = TESTING_DIR
        setup()
        db._reset()

    def tearDown(self):
        teardown()

    def test_missing_db(self):
        try:
            db.load()
        except FileNotFoundError:
            pass

        self.assertEqual(db.get().keys(), {}.keys())

    def test_malformed_db(self):
        with open(config.DB_DIR / config.DB_NAME, "tw") as db_file:
            db_file.write("{")

        with self.assertRaises(JSONDecodeError):
            db.load()

    def test_addition_removal(self):
        db.get(PROTON_DIR_1, PROTON_BIG).append(USER_DIR)

        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertIn(USER_DIR, db.get(PROTON_DIR_1, PROTON_BIG))

        db.get(PROTON_DIR_1, PROTON_BIG).remove(USER_DIR)

        self.assertIn(PROTON_BIG, db.get(PROTON_DIR_1))
        self.assertNotIn(USER_DIR, db.get(PROTON_DIR_1, PROTON_BIG))

        del db.get(PROTON_DIR_1)[PROTON_BIG]
        self.assertNotIn(PROTON_BIG, db.get(PROTON_DIR_1))
